/*******************************************************************
  Components for embedded applications builded for
  laboratory and medical instruments firmware  
 
  mx1_usb.c - USB support for DragonBall M9328 MX1 MCU
 
  Copyright (C) 2002-2004 Roman Bartosinski
            (C) 2004 by Pavel Pisa pisa@cmp.felk.cvut.cz
            (C) 2004 by PiKRON Ltd. http://www.pikron.com

 *******************************************************************/

#include <usb/usb.h>
#include <usb/usb_spec.h>
#include <usb/usbdebug.h>
#include <usb_mx1/mx1_usb_ll.h>
#include <mx1_reg.h>


/* init chip state */
  void usb_mx1_init_chip_state(usb_device_t *udev) {
    mx_usb_init_ll();
  }
 
/* connect usb */
  int usb_mx1_connect( usb_device_t *udev) {
    return 0;
  }

/* disconnect usb */
  int usb_mx1_disconnect( usb_device_t *udev) {
    return 0;
  }

/* acknowledge control transfer */
  void usb_mx1_ack_setup( usb_device_t *udev) {
    /*zero lenght packet should be generated there*/
  }


/* acknowledge control transfer */
  void usb_mx1_ack_control_setup( usb_device_t *udev) {
    /*zero lenght packet should be generated there*/
  }

  
/**
 * usb_mx1_check events
 * function reads interrupt register and sets event flags
 * function returns 1 if there is some new event.
 * function returns 0 if there isn't new event but all is OK
 * function returns -1 if there is any error
*/
  int usb_mx1_check_events( usb_device_t *udev) {
    unsigned int ep_int;
    unsigned int dev_int;
    int ret = 0, i;

    dev_int = _reg_USBD_INTR_STAT;
    _reg_USBD_INTR_STAT = dev_int;
   
    if (dev_int & MX1_USBD_INTR_SOF_m){
      _reg_USBD_CTRL |= MX1_USBD_CTRL_CMD_OVER_m;	// signal CMD_OVER
      dev_int &= ~MX1_USBD_INTR_SOF_m;
    }
    dev_int &= ~MX1_USBD_INTR_MSOF_m;			// We do not care for this

    if (dev_int) {
      usb_debug_print( DEBUG_LEVEL_LOW, ("USB INTR=0x%04X\n", dev_int));
    } else {
      //usb_debug_print( DEBUG_LEVEL_VERBOSE, ("USB No Int\n"));
    }
    
    if ( dev_int & MX1_USBD_INTR_RESET_START_m) {	// Bus reset reached
      mx_usb_reset();
      udev->flags |= USB_FLAG_BUS_RESET;
      ret = 1;
    } else {
      if ( dev_int & MX1_USBD_INTR_SUSP_m) {		// Suspend flag changed
        udev->flags |= USB_FLAG_SUSPEND;
        ret = 1;
      }

      if ( dev_int & MX1_USBD_INTR_WAKEUP_m) {		// Suspend flag changed
        udev->flags &= ~USB_FLAG_SUSPEND;
        ret = 1;
      }
            
      ep_int=_reg_USBD_EP(0).INTR_STAT;
      _reg_USBD_EP(0).INTR_STAT=ep_int;
      if(ep_int & MX1_EP_INTR_DEVREQ_m) {
        usb_debug_print( DEBUG_LEVEL_LOW, ("EP0 Drq INTR=0x%04X\n", ep_int));
        udev->flags |= USB_FLAG_SETUP;			// Ep0 SetupPacketReceived
	ret=1;
      } else {
        if(ep_int & MX1_EP_INTR_EOF_m) {
	  if(udev->ep0.flags&USB_STATE_RECEIVE) {
            usb_debug_print( DEBUG_LEVEL_LOW, ("EP0 Rx INTR=0x%04X\n", ep_int));
            udev->flags |= USB_FLAG_EVENT_RX0;		// Ep0 RxDone - some data was received in EP0
            ret = 1;
	  } else if(udev->ep0.flags&USB_STATE_TRANSMIT) {
            usb_debug_print( DEBUG_LEVEL_LOW, ("EP0 Tx INTR=0x%04X\n", ep_int));
            udev->flags |= USB_FLAG_EVENT_TX0;		// Ep0 TxDone - data in EP0 was sended
            ret = 1;
	  } else {
            _reg_USBD_EP(0).STAT=MX1_EP_STAT_FLUSH_m;
	  }
        }
      }
      
      for( i=0; i<udev->cntep; i++) {
        ep_int=_reg_USBD_EP(udev->ep[i].epnum).INTR_STAT;
        _reg_USBD_EP(udev->ep[i].epnum).INTR_STAT=ep_int;
        if (ep_int & MX1_EP_INTR_EOF_m) {
          udev->ep_events |= 1<<i;
          ret = 1;
        }
      }

    }
    return ret;
  }


/* stall endpoint X */
  void usb_mx1_stall( usb_ep_t *ep) {
    mx_usb_ep_stall(ep->epnum);
  }

  int usb_mx1_read_endpoint( usb_ep_t *ep, void *ptr, int size)
  {
    return mx_usb_ep_read(ep->epnum, ptr, size);
  }
 
  int usb_mx1_write_endpoint( usb_ep_t *ep, const void *ptr, int size)
  {
    return mx_usb_ep_write(ep->epnum, ptr, size);
  }

    
/* init usb structures and chip */
  int usb_mx1_init(usb_device_t *udev) {

   #ifndef USB_PDI_DIRECT_FNC
    udev->connect = usb_mx1_connect;
    udev->disconnect = usb_mx1_disconnect;
    udev->ack_setup = usb_mx1_ack_setup;
    udev->ack_control_setup = usb_mx1_ack_control_setup;
    udev->stall = usb_mx1_stall;
    udev->check_events = usb_mx1_check_events;
    udev->read_endpoint = usb_mx1_read_endpoint;
    udev->write_endpoint = usb_mx1_write_endpoint;
   #endif /*USB_PDI_DIRECT_FNC*/

    udev->ep0.max_packet_size = MX_USB_EP0_PACKET_SIZE;

    usb_mx1_init_chip_state(udev);
    return 0;
  }
